from revpayment.payments import NewebPayment, CreditPayment, EcpayPayment
from revpayment.settings import api_settings
import json


class CheckoutSDK:
    payment_classes = (
        ('neweb', NewebPayment),
        ('ecpay', EcpayPayment),
        ('credit', CreditPayment),
    )
    handler_class = api_settings.HANDLER_CLASS

    def __init__(self, state):
        self.state = state
        if type(state.cart) is str:
            self.cart = json.loads(state.cart)
        elif type(state.cart) is dict:
            self.cart = state.cart
        else:
            raise TypeError
        self.payment_type = state.payment_type
        self.payment_subtype = state.payment_subtype
        self.buyer = state.buyer

    def get_payment_class(self):
        try:
            payment_class = [
                c[1] for c in self.payment_classes if c[0] == self.payment_type][0]
            return payment_class
        except IndexError:
            raise ValueError('payment_type is not valid')

    def get_payment(self):
        payment_class = self.get_payment_class()
        return payment_class(buyer=self.buyer, payment_subtype=self.payment_subtype, cart=self.cart, order_id=self.state.order_id)

    def before_checkout(self):
        handler = self.handler_class(order=None)
        handler.before_checkout(state=self.state)

    def after_checkout(self):
        handler = self.handler_class(order=None)
        handler.after_checkout(state=self.state)

    def checkout(self):
        state = self.state
        self.before_checkout()
        payment = self.get_payment()
        result = payment.checkout(state)
        self.after_checkout()
        return result

    def callback(self, data):
        payment = self.get_payment()
        order, changed = payment.callback(self.state, data)
        handler = self.handler_class(order=order)
        handler.on_payment_result(changed)
        return f'{self.state.redirect_url}/order?id={order.id}'

    def customer_redirect(self, data):
        payment = self.get_payment()
        order = payment.customer_redirect(self.state, data)
        return f'{self.state.redirect_url}/order?id={order.id}'
